@@ -1,4 +1,6 @@
 #include "phantompy.hpp"
+#include "qjson/serializer.h"
+#include "qjson/parser.h"
 #include "private/cookiejar.hpp"
 #include "private/context.hpp"
 #include "private/page.hpp"
@@ -7,6 +9,20 @@
 #include "private/webelement.hpp"
 #include "private/webelementcollection.hpp"
 
+char* _bytearray_to_char_ptr(const QByteArray &data) {
+    char *resultData = new char[data.size() + 1];
+    qstrncpy(resultData, data.data(), data.size() + 1);
+
+    return resultData;
+}
+
+char* _qvariant_to_json_char_ptr(const QVariant &data) {
+    QJson::Serializer serializer;
+    bool ok;
+
+    return _bytearray_to_char_ptr(serializer.serialize(data, &ok));
+}
+
 extern "C" {
 
 /******* Context *******/
@@ -37,7 +53,6 @@ int32_t ph_context_get_boolean_config(int key) {
 
 int32_t ph_context_get_int_config(int key) {
     return ph::Context::instance()->getConfig(ph::Settings(key)).toInt();
-
 }
 
 void ph_context_clear_memory_cache() {
@@ -53,29 +68,19 @@ void ph_context_set_max_pages_in_cache(int num) {
 }
 
 char* ph_context_get_all_cookies() {
+    QJson::Serializer serializer;
+    bool ok;
+
     QVariantList cookies = ph::CookieJar::instance()->getAllCookies();
-
-    QJsonArray cookiesArray;
-
-    foreach(QVariant cvariant, cookies) {
-        QVariantMap cookie = cvariant.toMap();
-        QJsonObject cookieObj = QJsonObject::fromVariantMap(cookie);
-
-        cookiesArray.append(QJsonValue(cookieObj));
-    }
-
-    QByteArray data = QJsonDocument(cookiesArray).toJson();
-    char *resultData = new char[data.size() + 1];
-
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(serializer.serialize(cookies, &ok));
 }
 
 void ph_context_set_cookies(const char *cookies) {
-    QJsonArray cookiesArray = QJsonDocument::fromJson(QByteArray(cookies)).array();
-    QVariantList cookiesList = cookiesArray.toVariantList();
+    QJson::Parser parser;
+    bool ok;
 
-    ph::CookieJar::instance()->addCookiesFromMapList(cookiesList);
+    QVariant cookiesData = parser.parse(QByteArray(cookies), &ok);
+    ph::CookieJar::instance()->addCookiesFromMapList(cookiesData.toList());
 }
 
 void ph_context_clear_cookies() {
@@ -122,22 +127,25 @@ int ph_page_is_loaded(void *page) {
 
 void ph_page_set_initial_cookies(void *page, const char *cookies) {
     ph::Page *p = static_cast<ph::Page*>(page);
-    QJsonArray cookiesArray = QJsonDocument::fromJson(QByteArray(cookies)).array();
 
-    p->setInitialCookies(cookiesArray.toVariantList());
+    QJson::Parser parser;
+    bool ok;
+
+    QByteArray jsonData(cookies);
+    QVariant cookiesData = parser.parse(jsonData, &ok);
+
+    p->setInitialCookies(cookiesData.toList());
 }
 
 
 char* ph_page_get_cookies(void *page) {
     ph::Page *p = static_cast<ph::Page*>(page);
     QVariantList cookiesList = p->getCookies();
-    QJsonArray cookies = QJsonArray::fromVariantList(cookiesList);
 
-    QByteArray cookiesData = QJsonDocument(cookies).toJson();
-    char *resultData = new char[cookiesData.size() + 1];
+    QJson::Serializer serializer;
+    bool ok;
 
-    qstrncpy(resultData, cookiesData.data(), cookiesData.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(serializer.serialize(cookiesList, &ok));
 }
 
 
@@ -145,31 +153,45 @@ char* ph_page_get_requested_urls(void *page) {
     ph::Page *p = (ph::Page*)page;
     QSet<QString> urlsList = p->requestedUrls();
 
-    QJsonArray urls;
-    for(auto i=urlsList.cbegin(); i != urlsList.cend(); ++i) {
-        urls.append(QJsonValue((*i)));
+    QVariantList urls;
+    QSet<QString>::const_iterator i;
+
+#ifdef PHANTOMPY_QT4
+    for(i=urlsList.begin(); i != urlsList.end(); ++i) {
+#else
+    for(i=urlsList.cbegin(); i != urlsList.cend(); ++i) {
+#endif
+        urls << QVariant::fromValue((*i));
     }
 
-    QByteArray requestedUrls = QJsonDocument(urls).toJson();
-    char *resultData = new char[requestedUrls.size() + 1];
+    QJson::Serializer serializer;
+    bool ok;
 
-    qstrncpy(resultData, requestedUrls.data(), requestedUrls.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(serializer.serialize(urls, &ok));
 }
 
 char* ph_page_get_reply_by_url(void *page, const char *url) {
     ph::Page *p = static_cast<ph::Page*>(page);
     QVariantMap response = p->getResponseByUrl(QString::fromUtf8(url));
-    QJsonObject rsp = QJsonObject::fromVariantMap(response);
 
-    QByteArray data = QJsonDocument(rsp).toJson();
-    char *resultData = new char[data.size() + 1];
+    QJson::Serializer serializer;
+    bool ok;
 
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(serializer.serialize(response, &ok));
 }
 
-/****** Page ******/
+
+void ph_page_go_back(void *page) {
+    ph::Page *p = static_cast<ph::Page*>(page);
+    p->history()->back();
+}
+
+void ph_page_go_forward(void *page) {
+    ph::Page *p = static_cast<ph::Page*>(page);
+    p->history()->forward();
+}
+
+/****** Frame ******/
 
 void* ph_page_main_frame(void *page) {
     ph::Page *p = (ph::Page*)page;
@@ -184,22 +206,13 @@ void ph_frame_free(void *frame) {
 
 char* ph_frame_to_html(void *frame) {
     ph::Frame *f = (ph::Frame*)frame;
-
-    QByteArray data = f->toHtml().toUtf8();
-    char *resultData = new char[data.size() + 1];
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(f->toHtml().toUtf8());
 }
 
 char* ph_frame_evaluate_javascript(void *frame, char *javascript) {
     ph::Frame *f = (ph::Frame*)frame;
-
-    QString js(javascript);
-    QByteArray data = f->evaluateJavaScript(js).toUtf8();
-
-    char *resultData = new char[data.size() + 1];
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    QString js = QString::fromUtf8(javascript);
+    return _qvariant_to_json_char_ptr(f->evaluateJavaScript(js));
 }
 
 void* ph_frame_find_first(void *frame, const char *selector) {
@@ -217,6 +230,23 @@ void* ph_frame_find_all(void *frame, const char *selector) {
     return c;
 }
 
+char* ph_frame_get_url(void *frame) {
+    ph::Frame *f = static_cast<ph::Frame*>(frame);
+    return _bytearray_to_char_ptr(f->getUrl().toUtf8());
+}
+
+void ph_frame_set_url(void *frame, const char *url) {
+    ph::Frame *f = static_cast<ph::Frame*>(frame);
+    f->setUrl(QString::fromUtf8(url));
+}
+
+void ph_frame_load(void *frame, const char *url) {
+    ph::Frame *f = static_cast<ph::Frame*>(frame);
+    f->load(QString::fromUtf8(url));
+}
+
+/****** Web Element Collection *******/
+
 int32_t ph_webcollection_size(void *collection) {
     ph::WebElementCollection *c = static_cast<ph::WebElementCollection*>(collection);
     return c->size();
@@ -234,6 +264,8 @@ void ph_webcollection_free(void *collection) {
     delete c;
 }
 
+/******* Frame Image *********/
+
 void* ph_frame_capture_image(void *frame, const char *format, int quality) {
     ph::Frame *f = (ph::Frame*)frame;
     QByteArray image_data = f->captureImage(format, quality);
@@ -251,7 +283,7 @@ int64_t ph_image_get_size(void *image) {
     return img->size();
 }
 
-const char * ph_image_get_format(void *image) {
+const char* ph_image_get_format(void *image) {
     ph::Image *img = (ph::Image*)image;
     return img->format().data();
 }
@@ -282,59 +314,39 @@ void* ph_webelement_find_all(void *element, const char *selector) {
 
 char* ph_webelement_tag_name(void *element) {
     ph::WebElement *el = static_cast<ph::WebElement*>(element);
-
-    QByteArray data = el->tagName().toUtf8();
-
-    char *resultData = new char[data.size() + 1];
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(el->tagName().toUtf8());
 }
 
 char* ph_webelement_inner_html(void *element) {
     ph::WebElement *el = static_cast<ph::WebElement*>(element);
-    QByteArray data = el->toHtml().toUtf8();
-
-    char *resultData = new char[data.size() + 1];
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(el->toHtml().toUtf8());
 }
 
 char* ph_webelement_inner_text(void *element) {
     ph::WebElement *el = static_cast<ph::WebElement*>(element);
-    QByteArray data = el->toText().toUtf8();
-
-    char *resultData = new char[data.size() + 1];
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(el->toText().toUtf8());
 }
 
 
 char* ph_webelement_get_classes(void *element) {
     ph::WebElement *el = static_cast<ph::WebElement*>(element);
-    QByteArray data = el->getClasses().join(" ").toUtf8();
-
-    char *resultData = new char[data.size() + 1];
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    QByteArray data = el->getClasses().join(QString::fromUtf8(" ")).toUtf8();
 
+    return _bytearray_to_char_ptr(data);
 }
 
 char* ph_webelement_get_attrnames(void *element) {
     ph::WebElement *el = static_cast<ph::WebElement*>(element);
-    QByteArray data = el->getAttributeNames().join(" ").toUtf8();
+    QByteArray data = el->getAttributeNames().join(QString::fromUtf8(" ")).toUtf8();
 
-    char *resultData = new char[data.size() + 1];
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(data);
 }
 
 char* ph_webelement_get_attr(void *element, const char *attrname) {
     ph::WebElement *el = static_cast<ph::WebElement*>(element);
     QByteArray data = el->getAttribute(QString::fromUtf8(attrname)).toUtf8();
 
-    char *resultData = new char[data.size() + 1];
-    qstrncpy(resultData, data.data(), data.size() + 1);
-    return resultData;
+    return _bytearray_to_char_ptr(data);
 }
 
 int32_t ph_webelement_has_class(void *element, const char *classname) {
@@ -466,4 +478,10 @@ void* ph_webelement_next(void *element) {
     return new ph::WebElement(elm);
 }
 
+char* ph_webelement_evaluate_javascript(void *element, const char *javascript) {
+    ph::WebElement *el = static_cast<ph::WebElement*>(element);
+    QString js = QString::fromUtf8(javascript);
+    return _qvariant_to_json_char_ptr(el->evaluateJavaScript(js));
+}
+
 }
