@@ -2,7 +2,7 @@
 #include <cstring>
 
 extern "C" {
-    // Initialize methods
+    // Context methods (initialize)
     void* ph_context_init();
     void ph_context_free();
     void ph_free_charptr(char *ptr);
@@ -11,7 +11,7 @@ extern "C" {
     void ph_context_set_object_cache_capacity(int i, int x, int z);
     void ph_context_set_max_pages_in_cache(int num);
 
-    // CookieJar access
+    // CookieJar methods
     char* ph_context_get_all_cookies();
     void ph_context_set_cookies(const char *cookies);
     void ph_context_clear_cookies();
@@ -21,7 +21,7 @@ extern "C" {
     int32_t ph_context_get_boolean_config(int key);
     int32_t ph_context_get_int_config(int key);
 
-    // Page Methods
+    // Page methods
     void* ph_page_create();
     void ph_page_set_viewpoint_size(void *page, int x, int y);
     void ph_page_free(void *page);
@@ -32,25 +32,30 @@ extern "C" {
     int32_t ph_page_load(void *page, char *url);
     int32_t ph_page_is_loaded(void *page);
 
-    //char* ph_page_cookies(void *page);
+    void ph_page_go_back(void *page);
+    void ph_page_go_forward(void *page);
+
     char* ph_page_get_requested_urls(void *page);
     char* ph_page_get_reply_by_url(void *page, const char *url);
 
-    // Frame
+    // Frame methods
     void* ph_page_main_frame(void *page);
     void ph_frame_free(void *frame);
 
     char* ph_frame_to_html(void *frame);
-    char* ph_frame_evaluate_javascript(void *frame, char* js);
+    char* ph_frame_evaluate_javascript(void *frame, char* javascript);
+    char* ph_frame_get_url(void *frame);
+    void ph_frame_set_url(void *frame, const char *url);
+    void ph_frame_load(void *frame, const char *url);
 
-    // Image Methods
+    // Frame Image methods
     void* ph_frame_capture_image(void *frame, const char *format, int quality);
     void ph_image_free(void *image);
     int64_t ph_image_get_size(void* image);
     const char* ph_image_get_format(void* image);
     void ph_image_get_bytes(void *image, void *buffer, int64_t size);
 
-    // Web element methods
+    // Web Element and Web Element Collection methods
     void* ph_frame_find_first(void *frame, const char *selector); // TODO: deprectated
     void* ph_frame_find_all(void *frame, const char *selector);
     void* ph_webcollection_get_webelement(void *collection,  int32_t index);
@@ -68,6 +73,7 @@ extern "C" {
     char* ph_webelement_get_classes(void *element);
     char* ph_webelement_get_attnames(void *element);
     char* ph_webelement_get_attr(void *element, const char *attrname);
+    char* ph_webelement_evaluate_javascript(void *element, const char *javascript);
 
     int32_t ph_webcollection_size(void *collection);
     int32_t ph_webelement_has_class(void *element, const char *classname);
