@@ -3,7 +3,9 @@
 
 #include <QtCore>
 #include <QtWebKit>
+#ifndef PHANTOMPY_QT4
 #include <QtWebKitWidgets>
+#endif
 
 namespace ph {
 
@@ -16,11 +18,15 @@ public:
 
     QByteArray captureImage(const char *format, int quality);
     QString toHtml();
-    QString evaluateJavaScript(const QString &data);
+    QVariant evaluateJavaScript(const QString &data);
 
     QWebElement findFirstElement(const QString &selector);
     QWebElementCollection findAll(const QString &selector);
 
+    QString getUrl();
+    void setUrl(const QString &url);
+    void load(const QString &url);
+
 private:
     QWebFrame *p_frame;
 };
